import os
import subprocess
from pathlib import Path

import pytest


@pytest.fixture
def expanddbdtcl():
    require_path = Path(os.environ.get("E3_REQUIRE_LOCATION"))
    expanddbdtcl = require_path / "tools" / "expandDBD"
    assert expanddbdtcl.is_file() and os.access(expanddbdtcl, os.X_OK)
    return expanddbdtcl


def test_missing_dbd_file(tmp_path, expanddbdtcl):
    dbd_file = tmp_path / "tmp.dbd"
    dbd_file.write_text("include not_a_file.dbd")
    result = subprocess.run(
        [expanddbdtcl, "-I", tmp_path, dbd_file],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        encoding="utf-8",
    )
    assert result.returncode == 1
    assert "File 'not_a_file.dbd' not found" in result.stderr


def test_include_dbd_file(tmp_path, expanddbdtcl):
    dbd_a = tmp_path / "a.dbd"
    dbd_a.write_text("include b.dbd")

    dbd_b = tmp_path / "b.dbd"
    dbd_b.write_text("content")

    result = subprocess.run(
        [expanddbdtcl, "-I", str(tmp_path), dbd_a],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        encoding="utf-8",
    )
    assert result.returncode == 0
    assert "content" == result.stdout.strip("\n")


def test_skip_repeated_includes(tmp_path, expanddbdtcl):
    dbd_a = tmp_path / "a.dbd"
    dbd_a.write_text("include b.dbd\ninclude b.dbd")

    dbd_b = tmp_path / "b.dbd"
    dbd_b.touch()

    result = subprocess.run(
        [expanddbdtcl, "-I", str(tmp_path), dbd_a],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        encoding="utf-8",
    )
    assert result.returncode == 0
    assert "Info: skipping duplicate file b.dbd included from" in result.stderr


def test_do_not_skip_repeated_include_common_dbd(tmp_path, expanddbdtcl):
    dbd_a = tmp_path / "a.dbd"
    dbd_a.write_text("include dbCommon.dbd\ninclude dbCommon.dbd")

    dbd_b = tmp_path / "dbCommon.dbd"
    dbd_b.write_text("content")

    result = subprocess.run(
        [expanddbdtcl, "-I", str(tmp_path), dbd_a],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        encoding="utf-8",
    )
    assert result.returncode == 0
    assert result.stdout == "content\ncontent\n"


def test_record_names_from_dbds(tmp_path, expanddbdtcl):
    dbd_a = tmp_path / "a.dbd"
    dbd_a.write_text("include aRecord.dbd")

    dbd_record = tmp_path / "aRecord.dbd"
    dbd_record.touch()

    result = subprocess.run(
        [expanddbdtcl, "-r", "-I", str(tmp_path), dbd_a],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        encoding="utf-8",
    )
    assert result.returncode == 0
    assert result.stdout.splitlines() == ["a"]


def test_missing_record_names_from_dbds(tmp_path, expanddbdtcl):
    dbd_a = tmp_path / "a.dbd"
    dbd_a.write_text("include aRecord.dbd")

    result = subprocess.run(
        [expanddbdtcl, "-r", "-I", str(tmp_path), dbd_a],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        encoding="utf-8",
    )
    assert result.returncode == 1
